package sun.font;

import javax.swing.plaf.FontUIResource;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import sun.applet.AppletSecurity;
import java.util.Map;
import sun.misc.ThreadGroupUtils;
import sun.awt.AppContext;
import java.util.TreeMap;
import sun.awt.SunToolkit;
import java.awt.FontFormatException;
import java.security.AccessController;
import sun.util.logging.PlatformLogger;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.security.PrivilegedAction;
import java.util.Iterator;
import java.util.Hashtable;
import java.io.File;
import java.util.Vector;
import java.util.Locale;
import java.awt.Font;
import java.io.FilenameFilter;
import sun.awt.FontConfiguration;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.HashMap;
import java.util.concurrent.ConcurrentHashMap;
import sun.java2d.FontSupport;

public abstract class SunFontManager implements FontSupport, FontManagerForSGE
{
    public static final int FONTFORMAT_NONE = -1;
    public static final int FONTFORMAT_TRUETYPE = 0;
    public static final int FONTFORMAT_TYPE1 = 1;
    public static final int FONTFORMAT_T2K = 2;
    public static final int FONTFORMAT_TTC = 3;
    public static final int FONTFORMAT_COMPOSITE = 4;
    public static final int FONTFORMAT_NATIVE = 5;
    protected static final int CHANNELPOOLSIZE = 20;
    protected FileFont[] fontFileCache;
    private int lastPoolIndex;
    private int maxCompFont;
    private CompositeFont[] compFonts;
    private ConcurrentHashMap<String, CompositeFont> compositeFonts;
    private ConcurrentHashMap<String, PhysicalFont> physicalFonts;
    private ConcurrentHashMap<String, PhysicalFont> registeredFonts;
    protected ConcurrentHashMap<String, Font2D> fullNameToFont;
    private HashMap<String, TrueTypeFont> localeFullNamesToFont;
    private PhysicalFont defaultPhysicalFont;
    static boolean longAddresses;
    private boolean loaded1dot0Fonts;
    boolean loadedAllFonts;
    boolean loadedAllFontFiles;
    HashMap<String, String> jreFontMap;
    HashSet<String> jreLucidaFontFiles;
    String[] jreOtherFontFiles;
    boolean noOtherJREFontFiles;
    public static final String lucidaFontName = "Lucida Sans Regular";
    public static String jreLibDirName;
    public static String jreFontDirName;
    private static HashSet<String> missingFontFiles;
    private String defaultFontName;
    private String defaultFontFileName;
    protected HashSet registeredFontFiles;
    private ArrayList badFonts;
    protected String fontPath;
    private FontConfiguration fontConfig;
    private boolean discoveredAllFonts;
    private static final FilenameFilter ttFilter;
    private static final FilenameFilter t1Filter;
    private Font[] allFonts;
    private String[] allFamilies;
    private Locale lastDefaultLocale;
    public static boolean noType1Font;
    private static String[] STR_ARRAY;
    private boolean usePlatformFontMetrics;
    private final ConcurrentHashMap<String, SunFontManager.FontRegistrationInfo> deferredFontFiles;
    private final ConcurrentHashMap<String, Font2DHandle> initialisedFonts;
    private HashMap<String, String> fontToFileMap;
    private HashMap<String, String> fontToFamilyNameMap;
    private HashMap<String, ArrayList<String>> familyToFontListMap;
    private String[] pathDirs;
    private boolean haveCheckedUnreferencedFontFiles;
    static HashMap<String, SunFontManager.FamilyDescription> platformFontMap;
    private ConcurrentHashMap<String, Font2D> fontNameCache;
    protected Thread fileCloser;
    Vector<File> tmpFontFiles;
    private static final Object altJAFontKey;
    private static final Object localeFontKey;
    private static final Object proportionalFontKey;
    private boolean _usingPerAppContextComposites;
    private boolean _usingAlternateComposites;
    private static boolean gAltJAFont;
    private boolean gLocalePref;
    private boolean gPropPref;
    private static HashSet<String> installedNames;
    private static final Object regFamilyKey;
    private static final Object regFullNameKey;
    private Hashtable<String, FontFamily> createdByFamilyName;
    private Hashtable<String, Font2D> createdByFullName;
    private boolean fontsAreRegistered;
    private boolean fontsAreRegisteredPerAppContext;
    private static Locale systemLocale;
    
    public static SunFontManager getInstance() {
        return (SunFontManager)FontManagerFactory.getInstance();
    }
    
    public FilenameFilter getTrueTypeFilter() {
        return SunFontManager.ttFilter;
    }
    
    public FilenameFilter getType1Filter() {
        return SunFontManager.t1Filter;
    }
    
    @Override
    public boolean usingPerAppContextComposites() {
        return this._usingPerAppContextComposites;
    }
    
    private void initJREFontMap() {
        this.jreFontMap = new HashMap();
        this.jreLucidaFontFiles = new HashSet();
        if (isOpenJDK()) {
            return;
        }
        this.jreFontMap.put("lucida sans0", "LucidaSansRegular.ttf");
        this.jreFontMap.put("lucida sans1", "LucidaSansDemiBold.ttf");
        this.jreFontMap.put("lucida sans regular0", "LucidaSansRegular.ttf");
        this.jreFontMap.put("lucida sans regular1", "LucidaSansDemiBold.ttf");
        this.jreFontMap.put("lucida sans bold1", "LucidaSansDemiBold.ttf");
        this.jreFontMap.put("lucida sans demibold1", "LucidaSansDemiBold.ttf");
        this.jreFontMap.put("lucida sans typewriter0", "LucidaTypewriterRegular.ttf");
        this.jreFontMap.put("lucida sans typewriter1", "LucidaTypewriterBold.ttf");
        this.jreFontMap.put("lucida sans typewriter regular0", "LucidaTypewriter.ttf");
        this.jreFontMap.put("lucida sans typewriter regular1", "LucidaTypewriterBold.ttf");
        this.jreFontMap.put("lucida sans typewriter bold1", "LucidaTypewriterBold.ttf");
        this.jreFontMap.put("lucida sans typewriter demibold1", "LucidaTypewriterBold.ttf");
        this.jreFontMap.put("lucida bright0", "LucidaBrightRegular.ttf");
        this.jreFontMap.put("lucida bright1", "LucidaBrightDemiBold.ttf");
        this.jreFontMap.put("lucida bright2", "LucidaBrightItalic.ttf");
        this.jreFontMap.put("lucida bright3", "LucidaBrightDemiItalic.ttf");
        this.jreFontMap.put("lucida bright regular0", "LucidaBrightRegular.ttf");
        this.jreFontMap.put("lucida bright regular1", "LucidaBrightDemiBold.ttf");
        this.jreFontMap.put("lucida bright regular2", "LucidaBrightItalic.ttf");
        this.jreFontMap.put("lucida bright regular3", "LucidaBrightDemiItalic.ttf");
        this.jreFontMap.put("lucida bright bold1", "LucidaBrightDemiBold.ttf");
        this.jreFontMap.put("lucida bright bold3", "LucidaBrightDemiItalic.ttf");
        this.jreFontMap.put("lucida bright demibold1", "LucidaBrightDemiBold.ttf");
        this.jreFontMap.put("lucida bright demibold3", "LucidaBrightDemiItalic.ttf");
        this.jreFontMap.put("lucida bright italic2", "LucidaBrightItalic.ttf");
        this.jreFontMap.put("lucida bright italic3", "LucidaBrightDemiItalic.ttf");
        this.jreFontMap.put("lucida bright bold italic3", "LucidaBrightDemiItalic.ttf");
        this.jreFontMap.put("lucida bright demibold italic3", "LucidaBrightDemiItalic.ttf");
        Iterator<String> iterator = this.jreFontMap.values().iterator();
        while (iterator.hasNext()) {
            this.jreLucidaFontFiles.add((String)iterator.next());
        }
        return;
    }
    
    public TrueTypeFont getEUDCFont() {
        return null;
    }
    
    private static native void initIDs();
    
    protected SunFontManager() {
        this.fontFileCache = new FileFont[20];
        this.lastPoolIndex = 0;
        this.maxCompFont = 0;
        this.compFonts = new CompositeFont[20];
        this.compositeFonts = new ConcurrentHashMap();
        this.physicalFonts = new ConcurrentHashMap();
        this.registeredFonts = new ConcurrentHashMap();
        this.fullNameToFont = new ConcurrentHashMap();
        this.loaded1dot0Fonts = false;
        this.loadedAllFonts = false;
        this.loadedAllFontFiles = false;
        this.noOtherJREFontFiles = false;
        this.registeredFontFiles = new HashSet();
        this.discoveredAllFonts = false;
        this.usePlatformFontMetrics = false;
        this.deferredFontFiles = new ConcurrentHashMap();
        this.initialisedFonts = new ConcurrentHashMap();
        this.fontToFileMap = null;
        this.fontToFamilyNameMap = null;
        this.familyToFontListMap = null;
        this.pathDirs = null;
        this.fontNameCache = new ConcurrentHashMap();
        this.fileCloser = null;
        this.tmpFontFiles = null;
        this._usingPerAppContextComposites = false;
        this._usingAlternateComposites = false;
        this.gLocalePref = false;
        this.gPropPref = false;
        this.fontsAreRegistered = false;
        this.fontsAreRegisteredPerAppContext = false;
        this.initJREFontMap();
        AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                File file = new File(SunFontManager.jreFontDirName + File.separator + "badfonts.txt");
                if (file.exists()) {
                    FileInputStream in = null;
                    try {
                        SunFontManager.this.badFonts = new ArrayList();
                        in = new FileInputStream(file);
                        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(in));
                        while (true) {
                            String line = bufferedReader.readLine();
                            if (line == null) {
                                break;
                            }
                            if (FontUtilities.debugFonts()) {
                                FontUtilities.getLogger().warning("read bad font: " + line);
                            }
                            SunFontManager.this.badFonts.add(line);
                        }
                    }
                    catch (IOException ex) {
                        try {
                            if (in != null) {
                                in.close();
                            }
                        }
                        catch (IOException ex2) {}
                    }
                }
                if (FontUtilities.isLinux) {
                    SunFontManager.this.registerFontDir(SunFontManager.jreFontDirName);
                }
                SunFontManager.this.registerFontsInDir(SunFontManager.jreFontDirName, true, 2, true, false);
                SunFontManager.this.fontConfig = SunFontManager.this.createFontConfiguration();
                if (SunFontManager.isOpenJDK()) {
                    String[] defaultPlatformFont = SunFontManager.this.getDefaultPlatformFont();
                    SunFontManager.this.defaultFontName = defaultPlatformFont[0];
                    SunFontManager.this.defaultFontFileName = defaultPlatformFont[1];
                }
                String extraFontPath = SunFontManager.this.fontConfig.getExtraFontPath();
                boolean n = false;
                boolean n2 = false;
                String s = System.getProperty("sun.java2d.fontpath");
                if (s != null) {
                    if (s.startsWith("prepend:")) {
                        n = true;
                        s = s.substring("prepend:".length());
                    }
                    else if (s.startsWith("append:")) {
                        n2 = true;
                        s = s.substring("append:".length());
                    }
                }
                if (FontUtilities.debugFonts()) {
                    PlatformLogger logger = FontUtilities.getLogger();
                    logger.info("JRE font directory: " + SunFontManager.jreFontDirName);
                    logger.info("Extra font path: " + extraFontPath);
                    logger.info("Debug font path: " + s);
                }
                if (s != null) {
                    SunFontManager.this.fontPath = SunFontManager.this.getPlatformFontPath(SunFontManager.noType1Font);
                    if (extraFontPath != null) {
                        SunFontManager.this.fontPath = extraFontPath + File.pathSeparator + SunFontManager.this.fontPath;
                    }
                    if (!(n2 == 0)) {
                        SunFontManager.this.fontPath = SunFontManager.this.fontPath + File.pathSeparator + s;
                    }
                    else if (!(n == 0)) {
                        SunFontManager.this.fontPath = s + File.pathSeparator + SunFontManager.this.fontPath;
                    }
                    else {
                        SunFontManager.this.fontPath = s;
                    }
                    SunFontManager.this.registerFontDirs(SunFontManager.this.fontPath);
                }
                else if (extraFontPath != null) {
                    SunFontManager.this.registerFontDirs(extraFontPath);
                }
                if (FontUtilities.isSolaris && Locale.JAPAN.equals(Locale.getDefault())) {
                    SunFontManager.this.registerFontDir("/usr/openwin/lib/locale/ja/X11/fonts/TT");
                }
                SunFontManager.access$700(SunFontManager.this, SunFontManager.this.fontConfig, null);
                return null;
            }
        });
        if ((boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
            @Override
            public Boolean run() {
                String property = System.getProperty("java2d.font.usePlatformFont");
                String getenv = System.getenv("JAVA2D_USEPLATFORMFONT");
                return Boolean.valueOf("true".equals(property) || getenv != null);
            }
        })) {
            this.usePlatformFontMetrics = true;
            System.out.println("Enabling platform font metrics for win32. This is an unsupported option.");
            System.out.println("This yields incorrect composite font metrics as reported by 1.1.x releases.");
            System.out.println("It is appropriate only for use by applications which do not use any Java 2");
            System.out.println("functionality. This property will be removed in a later release.");
        }
    }
    
    @Override
    public Font2DHandle getNewComposite(String familyName, int style, Font2DHandle font2DHandle) {
        if (!(font2DHandle.font2D instanceof CompositeFont)) {
            return font2DHandle;
        }
        CompositeFont compositeFont = (CompositeFont)font2DHandle.font2D;
        PhysicalFont slotFont = compositeFont.getSlotFont(0);
        if (familyName == null) {
            familyName = slotFont.getFamilyName(null);
        }
        if (style == -1) {
            style = compositeFont.getStyle();
        }
        Font2D font2D = this.findFont2D(familyName, style, 0);
        if (!(font2D instanceof PhysicalFont)) {
            font2D = slotFont;
        }
        PhysicalFont physicalFont = (PhysicalFont)font2D;
        CompositeFont compositeFont2 = (CompositeFont)this.findFont2D("dialog", style, 0);
        if (compositeFont2 == null) {
            return font2DHandle;
        }
        return new Font2DHandle(new CompositeFont(physicalFont, compositeFont2));
    }
    
    protected void registerCompositeFont(String s, String[] array, String[] array2, int n, int[] array3, int[] array4, boolean b) {
        CompositeFont compositeFont = new CompositeFont(s, array, array2, n, array3, array4, b, this);
        this.addCompositeToFontList(compositeFont, 2);
        synchronized (this.compFonts) {
            this.compFonts[this.maxCompFont++] = compositeFont;
        }
    }
    
    protected static void registerCompositeFont(String s, String[] array, String[] array2, int n, int[] array3, int[] array4, boolean b, ConcurrentHashMap<String, Font2D> concurrentHashMap) {
        CompositeFont compositeFont = new CompositeFont(s, array, array2, n, array3, array4, b, getInstance());
        Font2D font2D = (Font2D)concurrentHashMap.get(s.toLowerCase(Locale.ENGLISH));
        if ((font2D instanceof CompositeFont)) {
            font2D.handle.font2D = compositeFont;
        }
        concurrentHashMap.put(s.toLowerCase(Locale.ENGLISH), compositeFont);
    }
    
    private void addCompositeToFontList(CompositeFont compositeFont, int n) {
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().info("Add to Family " + compositeFont.familyName + ", Font " + compositeFont.fullName + " rank=" + n);
        }
        compositeFont.setRank(n);
        this.compositeFonts.put(compositeFont.fullName, compositeFont);
        this.fullNameToFont.put(compositeFont.fullName.toLowerCase(Locale.ENGLISH), compositeFont);
        FontFamily family = FontFamily.getFamily(compositeFont.familyName);
        if (family == null) {
            family = new FontFamily(compositeFont.familyName, true, n);
        }
        family.setFont(compositeFont, compositeFont.style);
    }
    
    protected PhysicalFont addToFontList(PhysicalFont physicalFont, int i) {
        String fullName = physicalFont.fullName;
        String familyName = physicalFont.familyName;
        if (fullName == null || "".equals(fullName)) {
            return null;
        }
        if (this.compositeFonts.containsKey(fullName)) {
            return null;
        }
        physicalFont.setRank(i);
        if (!this.physicalFonts.containsKey(fullName)) {
            if (FontUtilities.isLogging()) {
                FontUtilities.getLogger().info("Add to Family " + familyName + ", Font " + fullName + " rank=" + i);
            }
            this.physicalFonts.put(fullName, physicalFont);
            FontFamily family2 = FontFamily.getFamily(familyName);
            if (family2 == null) {
                new FontFamily(familyName, false, i).setFont(physicalFont, physicalFont.style);
            }
            else {
                family2.setFont(physicalFont, physicalFont.style);
            }
            this.fullNameToFont.put(fullName.toLowerCase(Locale.ENGLISH), physicalFont);
            return physicalFont;
        }
        else {
            PhysicalFont physicalFont2 = (PhysicalFont)this.physicalFonts.get(fullName);
            if (physicalFont2 == null) {
                return null;
            }
            if (physicalFont2.getRank() >= i) {
                if (physicalFont2.mapper != null && i > 2) {
                    return physicalFont2;
                }
                if (physicalFont2.getRank() == i) {
                    if (physicalFont2 instanceof TrueTypeFont && physicalFont instanceof TrueTypeFont) {
                        if (((TrueTypeFont)physicalFont2).fileSize >= ((TrueTypeFont)physicalFont).fileSize) {
                            return physicalFont2;
                        }
                    }
                    else {
                        return physicalFont2;
                    }
                }
                if (physicalFont2.platName.startsWith(SunFontManager.jreFontDirName)) {
                    if (FontUtilities.isLogging()) {
                        FontUtilities.getLogger().warning("Unexpected attempt to replace a JRE  font " + fullName + " from " + physicalFont2.platName + " with " + physicalFont.platName);
                        return physicalFont2;
                    }
                    return physicalFont2;
                }
                else {
                    if (FontUtilities.isLogging()) {
                        FontUtilities.getLogger().info("Replace in Family " + familyName + ",Font " + fullName + " new rank=" + i + " from " + physicalFont2.platName + " with " + physicalFont.platName);
                    }
                    this.replaceFont(physicalFont2, physicalFont);
                    this.physicalFonts.put(fullName, physicalFont);
                    this.fullNameToFont.put(fullName.toLowerCase(Locale.ENGLISH), physicalFont);
                    FontFamily family = FontFamily.getFamily(familyName);
                    if (family == null) {
                        new FontFamily(familyName, false, i).setFont(physicalFont, physicalFont.style);
                        return physicalFont;
                    }
                    family.setFont(physicalFont, physicalFont.style);
                    return physicalFont;
                }
            }
            else {
                return physicalFont2;
            }
        }
    }
    
    public Font2D[] getRegisteredFonts() {
        PhysicalFont[] physicalFonts = this.getPhysicalFonts();
        int maxCompFont = this.maxCompFont;
        Font2D[] array = new Font2D[physicalFonts.length + maxCompFont];
        System.arraycopy(this.compFonts, 0, array, 0, maxCompFont);
        System.arraycopy(physicalFonts, 0, array, maxCompFont, physicalFonts.length);
        return array;
    }
    
    protected PhysicalFont[] getPhysicalFonts() {
        return (PhysicalFont[])this.physicalFonts.values().toArray(new PhysicalFont[0]);
    }
    
    protected synchronized void initialiseDeferredFonts() {
        Iterator<String> iterator = this.deferredFontFiles.keySet().iterator();
        while (iterator.hasNext()) {
            this.initialiseDeferredFont((String)iterator.next());
        }
    }
    
    protected synchronized void registerDeferredJREFonts(String prefix) {
        for (SunFontManager.FontRegistrationInfo fontRegistrationInfo : this.deferredFontFiles.values()) {
            if (fontRegistrationInfo.fontFilePath != null && fontRegistrationInfo.fontFilePath.startsWith(prefix)) {
                this.initialiseDeferredFont(fontRegistrationInfo.fontFilePath);
            }
            else {
                continue;
            }
        }
    }
    
    public boolean isDeferredFont(String key) {
        return this.deferredFontFiles.containsKey(key);
    }
    
    public PhysicalFont findJREDeferredFont(String s, int i) {
        String str = (String)this.jreFontMap.get(s.toLowerCase(Locale.ENGLISH) + i);
        if (str != null) {
            String string = SunFontManager.jreFontDirName + File.separator + str;
            if (this.deferredFontFiles.get(string) != null) {
                PhysicalFont initialiseDeferredFont = this.initialiseDeferredFont(string);
                if (initialiseDeferredFont != null && (initialiseDeferredFont.getFontName(null).equalsIgnoreCase(s) || initialiseDeferredFont.getFamilyName(null).equalsIgnoreCase(s)) && initialiseDeferredFont.style == i) {
                    return initialiseDeferredFont;
                }
            }
        }
        if (this.noOtherJREFontFiles) {
            return null;
        }
        synchronized (this.jreLucidaFontFiles) {
            if (this.jreOtherFontFiles == null) {
                HashSet o = new HashSet();
                for (String s2 : this.deferredFontFiles.keySet()) {
                    File o2 = new File(s2);
                    String parent = o2.getParent();
                    String name = o2.getName();
                    if (parent == null || !parent.equals(SunFontManager.jreFontDirName)) {
                        continue;
                    }
                    if (this.jreLucidaFontFiles.contains(name)) {
                        continue;
                    }
                    o.add(s2);
                }
                this.jreOtherFontFiles = (String[])o.toArray(SunFontManager.STR_ARRAY);
                if (this.jreOtherFontFiles.length == 0) {
                    this.noOtherJREFontFiles = true;
                }
            }
            for (int j = 0; j < this.jreOtherFontFiles.length; ++j) {
                String s3 = this.jreOtherFontFiles[j];
                if (s3 != null) {
                    this.jreOtherFontFiles[j] = null;
                    PhysicalFont initialiseDeferredFont2 = this.initialiseDeferredFont(s3);
                    if (initialiseDeferredFont2 != null && (initialiseDeferredFont2.getFontName(null).equalsIgnoreCase(s) || initialiseDeferredFont2.getFamilyName(null).equalsIgnoreCase(s)) && initialiseDeferredFont2.style == i) {
                        return initialiseDeferredFont2;
                    }
                }
            }
        }
        return null;
    }
    
    private PhysicalFont findOtherDeferredFont(String s, int n) {
        for (String pathname : this.deferredFontFiles.keySet()) {
            File o = new File(pathname);
            String parent = o.getParent();
            String name = o.getName();
            if (parent != null && parent.equals(SunFontManager.jreFontDirName) && this.jreLucidaFontFiles.contains(name)) {
                continue;
            }
            PhysicalFont initialiseDeferredFont = this.initialiseDeferredFont(pathname);
            if (initialiseDeferredFont != null && (initialiseDeferredFont.getFontName(null).equalsIgnoreCase(s) || initialiseDeferredFont.getFamilyName(null).equalsIgnoreCase(s)) && initialiseDeferredFont.style == n) {
                return initialiseDeferredFont;
            }
            continue;
        }
        return null;
    }
    
    private PhysicalFont findDeferredFont(String s, int n) {
        PhysicalFont jreDeferredFont = this.findJREDeferredFont(s, n);
        if (jreDeferredFont != null) {
            return jreDeferredFont;
        }
        return this.findOtherDeferredFont(s, n);
    }
    
    public void registerDeferredFont(String key, String s, String[] array, int n, boolean b, int n2) {
        this.deferredFontFiles.put(key, new SunFontManager.FontRegistrationInfo(s, array, n, b, n2));
    }
    
    public synchronized PhysicalFont initialiseDeferredFont(String s) {
        if (s == null) {
            return null;
        }
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().info("Opening deferred font file " + s);
        }
        SunFontManager.FontRegistrationInfo fontRegistrationInfo = (SunFontManager.FontRegistrationInfo)this.deferredFontFiles.get(s);
        PhysicalFont physicalFont;
        if (fontRegistrationInfo != null) {
            this.deferredFontFiles.remove(s);
            physicalFont = this.registerFontFile(fontRegistrationInfo.fontFilePath, fontRegistrationInfo.nativeNames, fontRegistrationInfo.fontFormat, fontRegistrationInfo.javaRasterizer, fontRegistrationInfo.fontRank);
            if (physicalFont != null) {
                this.initialisedFonts.put(s, physicalFont.handle);
            }
            else {
                this.initialisedFonts.put(s, this.getDefaultPhysicalFont().handle);
            }
        }
        else {
            Font2DHandle font2DHandle = (Font2DHandle)this.initialisedFonts.get(s);
            if (font2DHandle == null) {
                physicalFont = this.getDefaultPhysicalFont();
            }
            else {
                physicalFont = (PhysicalFont)(PhysicalFont)font2DHandle.font2D;
            }
        }
        return physicalFont;
    }
    
    public boolean isRegisteredFontFile(String key) {
        return this.registeredFonts.containsKey(key);
    }
    
    public PhysicalFont getRegisteredFontFile(String key) {
        return (PhysicalFont)this.registeredFonts.get(key);
    }
    
    public PhysicalFont registerFontFile(String s, String[] array, int n, boolean b, int i) {
        PhysicalFont physicalFont = (PhysicalFont)this.registeredFonts.get(s);
        if (physicalFont != null) {
            return physicalFont;
        }
        PhysicalFont physicalFont2 = null;
        try {
            switch (n) {
                case 0: {
                    int j = 0;
                    Object o;
                    do {
                        o = new TrueTypeFont(s, array, j++, b);
                        PhysicalFont addToFontList = this.addToFontList(o, i);
                        if (physicalFont2 == null) {
                            physicalFont2 = addToFontList;
                        }
                        else {
                            continue;
                        }
                    } while (j < o.getFontCount());
                    break;
                }
                case 1: {
                    physicalFont2 = this.addToFontList(new Type1Font(s, array), i);
                    break;
                }
                case 5: {
                    physicalFont2 = this.addToFontList(new NativeFont(s, false), i);
                    break;
                }
            }
            if (FontUtilities.isLogging()) {
                FontUtilities.getLogger().info("Registered file " + s + " as font " + physicalFont2 + " rank=" + i);
            }
        }
        catch (FontFormatException ex) {
            if (FontUtilities.isLogging()) {
                FontUtilities.getLogger().warning("Unusable font: " + s + " " + ex.toString());
            }
        }
        if (physicalFont2 != null && n != 5) {
            this.registeredFonts.put(s, physicalFont2);
            return physicalFont2;
        }
        return physicalFont2;
    }
    
    public void registerFonts(String[] array, String[][] array2, int n, int n2, boolean b, int n3, boolean b2) {
        for (int i = 0; i < n; ++i) {
            if (b2) {
                this.registerDeferredFont(array[i], array[i], array2[i], n2, b, n3);
            }
            else {
                this.registerFontFile(array[i], array2[i], n2, b, n3);
            }
        }
    }
    
    public PhysicalFont getDefaultPhysicalFont() {
        if (this.defaultPhysicalFont == null) {
            this.defaultPhysicalFont = (PhysicalFont)this.findFont2D("Lucida Sans Regular", 0, 0);
            if (this.defaultPhysicalFont == null) {
                this.defaultPhysicalFont = (PhysicalFont)this.findFont2D("Arial", 0, 0);
            }
            if (this.defaultPhysicalFont == null) {
                Iterator<PhysicalFont> iterator = this.physicalFonts.values().iterator();
                if (iterator.hasNext()) {
                    this.defaultPhysicalFont = (PhysicalFont)iterator.next();
                }
                else {
                    throw new Error("Probable fatal error:No fonts found.");
                }
            }
        }
        return this.defaultPhysicalFont;
    }
    
    public Font2D getDefaultLogicalFont(int n) {
        return this.findFont2D("dialog", n, 0);
    }
    
    private static String dotStyleStr(int n) {
        switch (n) {
            case 1: {
                return ".bold";
            }
            case 2: {
                return ".italic";
            }
            case 3: {
                return ".bolditalic";
            }
            default: {
                return ".plain";
            }
        }
    }
    
    protected void populateFontFileNameMap(HashMap<String, String> hashMap, HashMap<String, String> hashMap2, HashMap<String, ArrayList<String>> hashMap3, Locale locale) {
    }
    
    private String[] getFontFilesFromPath(boolean b) {
        final FilenameFilter ttFilter;
        if (b) {
            ttFilter = SunFontManager.ttFilter;
        }
        else {
            ttFilter = new SunFontManager.TTorT1Filter();
        }
        return (String[])(String[])AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                if (SunFontManager.this.pathDirs.length == 1) {
                    String[] list3 = new File(SunFontManager.this.pathDirs[0]).list(ttFilter);
                    if (list3 == null) {
                        return new String[0];
                    }
                    for (int k = 0; k < list3.length; ++k) {
                        list3[k] = list3[k].toLowerCase();
                    }
                    return list3;
                }
                else {
                    ArrayList<String> list = new ArrayList();
                    for (int i = 0; i < SunFontManager.this.pathDirs.length; ++i) {
                        String[] list2 = new File(SunFontManager.this.pathDirs[i]).list(ttFilter);
                        if (list2 != null) {
                            for (int j = 0; j < list2.length; ++j) {
                                list.add(list2[j].toLowerCase());
                            }
                        }
                    }
                    return list.toArray(SunFontManager.STR_ARRAY);
                }
            }
        });
    }
    
    private void resolveWindowsFonts() {
        Object o = null;
        for (String s : this.fontToFamilyNameMap.keySet()) {
            if ((String)this.fontToFileMap.get(s) == null) {
                if (s.indexOf("  ") > 0) {
                    String replaceFirst = s.replaceFirst("  ", " ");
                    String value2 = (String)this.fontToFileMap.get(replaceFirst);
                    if (value2 != null && !this.fontToFamilyNameMap.containsKey(replaceFirst)) {
                        this.fontToFileMap.remove(replaceFirst);
                        this.fontToFileMap.put(s, value2);
                    }
                    else {
                        continue;
                    }
                }
                else if (s.equals("marlett")) {
                    this.fontToFileMap.put(s, "marlett.ttf");
                }
                else if (s.equals("david")) {
                    String value = (String)this.fontToFileMap.get("david regular");
                    if (value != null) {
                        this.fontToFileMap.remove("david regular");
                        this.fontToFileMap.put("david", value);
                    }
                    else {
                        continue;
                    }
                }
                else {
                    if (o == null) {
                        o = new ArrayList();
                    }
                    o.add(s);
                }
            }
            else {
                continue;
            }
        }
        if (o != null) {
            HashSet o2 = new HashSet();
            HashMap<Object, Object> hashMap = (HashMap)(HashMap)this.fontToFileMap.clone();
            Iterator<String> iterator2 = this.fontToFamilyNameMap.keySet().iterator();
            while (iterator2.hasNext()) {
                hashMap.remove((Object)iterator2.next());
            }
            for (String s2 : hashMap.keySet()) {
                o2.add(hashMap.get(s2));
                this.fontToFileMap.remove(s2);
            }
            this.resolveFontFiles(o2, o);
            if (o.size() > 0) {
                ArrayList o3 = new ArrayList();
                Iterator<String> iterator4 = this.fontToFileMap.values().iterator();
                while (iterator4.hasNext()) {
                    o3.add(((String)iterator4.next()).toLowerCase());
                }
                for (String s3 : this.getFontFilesFromPath(true)) {
                    if (!o3.contains(s3)) {
                        o2.add(s3);
                    }
                }
                this.resolveFontFiles(o2, o);
            }
            if (o.size() > 0) {
                for (int size = o.size(), j = 0; j < size; ++j) {
                    String str = (String)o.get(j);
                    String s4 = (String)this.fontToFamilyNameMap.get(str);
                    if (s4 != null) {
                        ArrayList<Object> list = (ArrayList)this.familyToFontListMap.get(s4);
                        if (list != null && list.size() <= 1) {
                            this.familyToFontListMap.remove(s4);
                        }
                    }
                    this.fontToFamilyNameMap.remove(str);
                    if (FontUtilities.isLogging()) {
                        FontUtilities.getLogger().info("No file for font:" + str);
                    }
                }
            }
        }
    }
    
    private synchronized void checkForUnreferencedFontFiles() {
        if (this.haveCheckedUnreferencedFontFiles) {
            return;
        }
        this.haveCheckedUnreferencedFontFiles = true;
        if (!FontUtilities.isWindows) {
            return;
        }
        ArrayList<String> list = new ArrayList();
        Iterator<String> iterator = this.fontToFileMap.values().iterator();
        while (iterator.hasNext()) {
            list.add(((String)iterator.next()).toLowerCase());
        }
        Object fontToFileMap = null;
        Object fontToFamilyNameMap = null;
        Object familyToFontListMap = null;
        for (String value : this.getFontFilesFromPath(false)) {
            if (!list.contains(value)) {
                if (FontUtilities.isLogging()) {
                    FontUtilities.getLogger().info("Found non-registry file : " + value);
                }
                PhysicalFont registerFontFile = this.registerFontFile(this.getPathName(value));
                if (registerFontFile != null) {
                    if (fontToFileMap == null) {
                        fontToFileMap = new HashMap(this.fontToFileMap);
                        fontToFamilyNameMap = new HashMap(this.fontToFamilyNameMap);
                        familyToFontListMap = new HashMap(this.familyToFontListMap);
                    }
                    String fontName = registerFontFile.getFontName(null);
                    String familyName = registerFontFile.getFamilyName(null);
                    String lowerCase = familyName.toLowerCase();
                    fontToFamilyNameMap.put(fontName, familyName);
                    fontToFileMap.put(fontName, value);
                    ArrayList<Object> c = (ArrayList)familyToFontListMap.get(lowerCase);
                    Object value2;
                    if (c == null) {
                        value2 = new ArrayList();
                    }
                    else {
                        value2 = new ArrayList(c);
                    }
                    value2.add(fontName);
                    familyToFontListMap.put(lowerCase, value2);
                }
            }
        }
        if (fontToFileMap != null) {
            this.fontToFileMap = fontToFileMap;
            this.familyToFontListMap = familyToFontListMap;
            this.fontToFamilyNameMap = fontToFamilyNameMap;
        }
        return;
    }
    
    private void resolveFontFiles(HashSet<String> set, ArrayList<String> list) {
        Locale startupLocale = SunToolkit.getStartupLocale();
        for (String value : set) {
            try {
                int i = 0;
                String pathName = this.getPathName(value);
                if (FontUtilities.isLogging()) {
                    FontUtilities.getLogger().info("Trying to resolve file " + pathName);
                }
                Object o;
                do {
                    o = new TrueTypeFont(pathName, null, i++, false);
                    String lowerCase = o.getFontName(startupLocale).toLowerCase();
                    if (list.contains(lowerCase)) {
                        this.fontToFileMap.put(lowerCase, value);
                        list.remove(lowerCase);
                        if (FontUtilities.isLogging()) {
                            FontUtilities.getLogger().info("Resolved absent registry entry for " + lowerCase + " located in " + pathName);
                        }
                        else {
                            continue;
                        }
                    }
                    else {
                        continue;
                    }
                } while (i < o.getFontCount());
            }
            catch (Exception ex) {}
        }
    }
    
    public HashMap<String, SunFontManager.FamilyDescription> populateHardcodedFileNameMap() {
        return new HashMap(0);
    }
    
    Font2D findFontFromPlatformMap(String substring, int n) {
        if (SunFontManager.platformFontMap == null) {
            SunFontManager.platformFontMap = this.populateHardcodedFileNameMap();
        }
        if (SunFontManager.platformFontMap == null || SunFontManager.platformFontMap.size() == 0) {
            return null;
        }
        int index = substring.indexOf(32);
        if (index > 0) {
            substring = substring.substring(0, index);
        }
        SunFontManager.FamilyDescription familyDescription = (SunFontManager.FamilyDescription)SunFontManager.platformFontMap.get(substring);
        if (familyDescription == null) {
            return null;
        }
        int n2 = -1;
        if (substring.equalsIgnoreCase(familyDescription.plainFullName)) {
            n2 = 0;
        }
        else if (substring.equalsIgnoreCase(familyDescription.boldFullName)) {
            n2 = 1;
        }
        else if (substring.equalsIgnoreCase(familyDescription.italicFullName)) {
            n2 = 2;
        }
        else if (substring.equalsIgnoreCase(familyDescription.boldItalicFullName)) {
            n2 = 3;
        }
        if (n2 == -1 && !substring.equalsIgnoreCase(familyDescription.familyName)) {
            return null;
        }
        String pathName = null;
        String pathName2 = null;
        String pathName3 = null;
        String pathName4 = null;
        boolean n3 = false;
        this.getPlatformFontDirs(SunFontManager.noType1Font);
        if (familyDescription.plainFileName != null) {
            pathName = this.getPathName(familyDescription.plainFileName);
            if (pathName == null) {
                n3 = true;
            }
        }
        if (familyDescription.boldFileName != null) {
            pathName2 = this.getPathName(familyDescription.boldFileName);
            if (pathName2 == null) {
                n3 = true;
            }
        }
        if (familyDescription.italicFileName != null) {
            pathName3 = this.getPathName(familyDescription.italicFileName);
            if (pathName3 == null) {
                n3 = true;
            }
        }
        if (familyDescription.boldItalicFileName != null) {
            pathName4 = this.getPathName(familyDescription.boldItalicFileName);
            if (pathName4 == null) {
                n3 = true;
            }
        }
        if (!(n3 == 0)) {
            if (FontUtilities.isLogging()) {
                FontUtilities.getLogger().info("Hardcoded file missing looking for " + substring);
            }
            SunFontManager.platformFontMap.remove(substring);
            return null;
        }
        else {
            final String[] array = { pathName, pathName2, pathName3, pathName4 };
            if ((boolean)AccessController.doPrivileged(new PrivilegedAction<Boolean>() {
                @Override
                public Boolean run() {
                    for (int i = 0; i < array.length; ++i) {
                        if (array[i] != null && !new File(array[i]).exists()) {
                            return Boolean.TRUE;
                        }
                    }
                    return Boolean.FALSE;
                }
            })) {
                if (FontUtilities.isLogging()) {
                    FontUtilities.getLogger().info("Hardcoded file missing looking for " + substring);
                }
                SunFontManager.platformFontMap.remove(substring);
                return null;
            }
            else {
                Font2D font2D = null;
                for (int i = 0; i < array.length; ++i) {
                    if (array[i] != null) {
                        PhysicalFont registerFontFile = this.registerFontFile(array[i], null, 0, false, 3);
                        if (i == n2) {
                            font2D = registerFontFile;
                        }
                    }
                }
                FontFamily family = FontFamily.getFamily(familyDescription.familyName);
                if (family != null) {
                    if (font2D == null) {
                        font2D = family.getFont(n);
                        if (font2D == null) {
                            font2D = family.getClosestStyle(n);
                        }
                    }
                    else if (n > 0 && n != font2D.style) {
                        n |= font2D.style;
                        font2D = family.getFont(n);
                        if (font2D == null) {
                            font2D = family.getClosestStyle(n);
                        }
                    }
                }
                return font2D;
            }
        }
    }
    
    private synchronized HashMap<String, String> getFullNameToFileMap() {
        if (this.fontToFileMap == null) {
            this.pathDirs = this.getPlatformFontDirs(SunFontManager.noType1Font);
            this.fontToFileMap = new HashMap(100);
            this.fontToFamilyNameMap = new HashMap(100);
            this.familyToFontListMap = new HashMap(50);
            this.populateFontFileNameMap(this.fontToFileMap, this.fontToFamilyNameMap, this.familyToFontListMap, Locale.ENGLISH);
            if (FontUtilities.isWindows) {
                this.resolveWindowsFonts();
            }
            if (FontUtilities.isLogging()) {
                this.logPlatformFontInfo();
            }
        }
        return this.fontToFileMap;
    }
    
    private void logPlatformFontInfo() {
        PlatformLogger logger = FontUtilities.getLogger();
        for (int i = 0; i < this.pathDirs.length; ++i) {
            logger.info("fontdir=" + this.pathDirs[i]);
        }
        for (String s : this.fontToFileMap.keySet()) {
            logger.info("font=" + s + " file=" + (String)this.fontToFileMap.get(s));
        }
        for (String s2 : this.fontToFamilyNameMap.keySet()) {
            logger.info("font=" + s2 + " family=" + (String)this.fontToFamilyNameMap.get(s2));
        }
        for (String s3 : this.familyToFontListMap.keySet()) {
            logger.info("family=" + s3 + " fonts=" + this.familyToFontListMap.get(s3));
        }
    }
    
    protected String[] getFontNamesFromPlatform() {
        if (this.getFullNameToFileMap().size() == 0) {
            return null;
        }
        this.checkForUnreferencedFontFiles();
        ArrayList<Object> list = new ArrayList();
        Iterator<ArrayList<String>> iterator = this.familyToFontListMap.values().iterator();
        while (iterator.hasNext()) {
            Iterator<Object> iterator2 = ((ArrayList)iterator.next()).iterator();
            while (iterator2.hasNext()) {
                list.add((Object)iterator2.next());
            }
        }
        return (String[])list.toArray(SunFontManager.STR_ARRAY);
    }
    
    public boolean gotFontsFromPlatform() {
        return this.getFullNameToFileMap().size() != 0;
    }
    
    public String getFileNameForFontName(String s) {
        return (String)this.fontToFileMap.get(s.toLowerCase(Locale.ENGLISH));
    }
    
    private PhysicalFont registerFontFile(String s) {
        if (new File(s).isAbsolute() && !this.registeredFonts.contains(s)) {
            int n = -1;
            int n2 = 6;
            if (SunFontManager.ttFilter.accept(null, s)) {
                n = 0;
                n2 = 3;
            }
            else if (SunFontManager.t1Filter.accept(null, s)) {
                n = 1;
                n2 = 4;
            }
            if (n == -1) {
                return null;
            }
            return this.registerFontFile(s, null, n, false, n2);
        }
        else {
            return null;
        }
    }
    
    protected void registerOtherFontFiles(HashSet set) {
        if (this.getFullNameToFileMap().size() == 0) {
            return;
        }
        Iterator<String> iterator = this.fontToFileMap.values().iterator();
        while (iterator.hasNext()) {
            this.registerFontFile((String)iterator.next());
        }
        return;
    }
    
    public boolean getFamilyNamesFromPlatform(TreeMap<String, String> treeMap, Locale locale) {
        if (this.getFullNameToFileMap().size() == 0) {
            return false;
        }
        this.checkForUnreferencedFontFiles();
        for (String value : this.fontToFamilyNameMap.values()) {
            treeMap.put(value.toLowerCase(locale), value);
        }
        return true;
    }
    
    private String getPathName(final String s) {
        if (new File(s).isAbsolute()) {
            return s;
        }
        if (this.pathDirs.length == 1) {
            return this.pathDirs[0] + File.separator + s;
        }
        String s2 = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                int i = 0;
                while (i < SunFontManager.this.pathDirs.length) {
                    File o = new File(SunFontManager.this.pathDirs[i] + File.separator + s);
                    if (o.exists()) {
                        return o.getAbsolutePath();
                    }
                    ++i;
                }
                return null;
            }
        });
        if (s2 != null) {
            return s2;
        }
        return s;
    }
    
    private Font2D findFontFromPlatform(String key, int n) {
        if (this.getFullNameToFileMap().size() == 0) {
            return null;
        }
        String s = null;
        String str = (String)this.fontToFamilyNameMap.get(key);
        ArrayList<Object> list;
        if (str != null) {
            s = (String)this.fontToFileMap.get(key);
            list = (ArrayList)this.familyToFontListMap.get(str.toLowerCase(Locale.ENGLISH));
        }
        else {
            list = (ArrayList)this.familyToFontListMap.get(key);
            if (list != null && list.size() > 0) {
                String lowerCase = ((String)list.get(0)).toLowerCase(Locale.ENGLISH);
                if (lowerCase != null) {
                    str = (String)this.fontToFamilyNameMap.get(lowerCase);
                }
            }
        }
        if (list == null || str == null) {
            return null;
        }
        String[] array = (String[])(String[])list.toArray(SunFontManager.STR_ARRAY);
        if (array.length == 0) {
            return null;
        }
        int i = 0;
        while (i < array.length) {
            if ((String)this.fontToFileMap.get(array[i].toLowerCase(Locale.ENGLISH)) == null) {
                if (FontUtilities.isLogging()) {
                    FontUtilities.getLogger().info("Platform lookup : No file for font " + array[i] + " in family " + str);
                }
                return null;
            }
            else {
                ++i;
            }
        }
        PhysicalFont registerFontFile = null;
        if (s != null) {
            registerFontFile = this.registerFontFile(this.getPathName(s), null, 0, false, 3);
        }
        for (int j = 0; j < array.length; ++j) {
            String anObject = (String)this.fontToFileMap.get(array[j].toLowerCase(Locale.ENGLISH));
            if (s == null || !s.equals(anObject)) {
                this.registerFontFile(this.getPathName(anObject), null, 0, false, 3);
            }
        }
        Font2D font2D = null;
        FontFamily family = FontFamily.getFamily(str);
        if (registerFontFile != null) {
            n |= registerFontFile.style;
        }
        if (family != null) {
            font2D = family.getFont(n);
            if (font2D == null) {
                font2D = family.getClosestStyle(n);
            }
        }
        return font2D;
    }
    
    @Override
    public Font2D findFont2D(String str, int i, int n) {
        String lowerCase = str.toLowerCase(Locale.ENGLISH);
        String string = lowerCase + dotStyleStr(i);
        Font2D font2D;
        if (this._usingPerAppContextComposites) {
            ConcurrentHashMap<Object, Object> concurrentHashMap = (ConcurrentHashMap)AppContext.getAppContext().get(CompositeFont.class);
            if (concurrentHashMap != null) {
                font2D = (Font2D)concurrentHashMap.get(string);
            }
            else {
                font2D = null;
            }
        }
        else {
            font2D = (Font2D)this.fontNameCache.get(string);
        }
        if (font2D != null) {
            return font2D;
        }
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().info("Search for font: " + str);
        }
        if (FontUtilities.isWindows) {
            if (lowerCase.equals("ms sans serif")) {
                str = "sansserif";
            }
            else if (lowerCase.equals("ms serif")) {
                str = "serif";
            }
        }
        if (lowerCase.equals("default")) {
            str = "dialog";
        }
        FontFamily family = FontFamily.getFamily(str);
        if (family != null) {
            Font2D value = family.getFontWithExactStyleMatch(i);
            if (value == null) {
                value = this.findDeferredFont(str, i);
            }
            if (value == null) {
                value = family.getFont(i);
            }
            if (value == null) {
                value = family.getClosestStyle(i);
            }
            if (value != null) {
                this.fontNameCache.put(string, value);
                return value;
            }
        }
        Font2D value2 = (Font2D)this.fullNameToFont.get(lowerCase);
        if (value2 != null) {
            if (value2.style == i || i == 0) {
                this.fontNameCache.put(string, value2);
                return value2;
            }
            FontFamily family2 = FontFamily.getFamily(value2.getFamilyName(null));
            if (family2 != null) {
                Font2D font = family2.getFont(i | value2.style);
                if (font != null) {
                    this.fontNameCache.put(string, font);
                    return font;
                }
                Font2D closestStyle = family2.getClosestStyle(i | value2.style);
                if (closestStyle != null && closestStyle.canDoStyle(i | value2.style)) {
                    this.fontNameCache.put(string, closestStyle);
                    return closestStyle;
                }
            }
        }
        if (FontUtilities.isWindows) {
            Font2D fontFromPlatformMap = this.findFontFromPlatformMap(lowerCase, i);
            if (FontUtilities.isLogging()) {
                FontUtilities.getLogger().info("findFontFromPlatformMap returned " + fontFromPlatformMap);
            }
            if (fontFromPlatformMap != null) {
                this.fontNameCache.put(string, fontFromPlatformMap);
                return fontFromPlatformMap;
            }
            if (this.deferredFontFiles.size() > 0) {
                PhysicalFont jreDeferredFont = this.findJREDeferredFont(lowerCase, i);
                if (jreDeferredFont != null) {
                    this.fontNameCache.put(string, jreDeferredFont);
                    return jreDeferredFont;
                }
            }
            Font2D fontFromPlatform = this.findFontFromPlatform(lowerCase, i);
            if (fontFromPlatform != null) {
                if (FontUtilities.isLogging()) {
                    FontUtilities.getLogger().info("Found font via platform API for request:\"" + str + "\":, style=" + i + " found font: " + fontFromPlatform);
                }
                this.fontNameCache.put(string, fontFromPlatform);
                return fontFromPlatform;
            }
        }
        if (this.deferredFontFiles.size() > 0) {
            PhysicalFont deferredFont = this.findDeferredFont(str, i);
            if (deferredFont != null) {
                this.fontNameCache.put(string, deferredFont);
                return deferredFont;
            }
        }
        if (FontUtilities.isSolaris && !this.loaded1dot0Fonts) {
            if (lowerCase.equals("timesroman")) {
                this.fontNameCache.put(string, this.findFont2D("serif", i, n));
            }
            this.register1dot0Fonts();
            this.loaded1dot0Fonts = true;
            return this.findFont2D(str, i, n);
        }
        else {
            if (this.fontsAreRegistered || this.fontsAreRegisteredPerAppContext) {
                Hashtable<Object, Object> createdByFamilyName;
                Hashtable<Object, Object> createdByFullName;
                if (this.fontsAreRegistered) {
                    createdByFamilyName = this.createdByFamilyName;
                    createdByFullName = this.createdByFullName;
                }
                else {
                    AppContext appContext = AppContext.getAppContext();
                    createdByFamilyName = (Hashtable)appContext.get(SunFontManager.regFamilyKey);
                    createdByFullName = (Hashtable)appContext.get(SunFontManager.regFullNameKey);
                }
                FontFamily fontFamily = (FontFamily)createdByFamilyName.get(lowerCase);
                if (fontFamily != null) {
                    Font2D value3 = fontFamily.getFontWithExactStyleMatch(i);
                    if (value3 == null) {
                        value3 = fontFamily.getFont(i);
                    }
                    if (value3 == null) {
                        value3 = fontFamily.getClosestStyle(i);
                    }
                    if (value3 != null) {
                        if (this.fontsAreRegistered) {
                            this.fontNameCache.put(string, value3);
                            return value3;
                        }
                        return value3;
                    }
                }
                Font2D value4 = (Font2D)createdByFullName.get(lowerCase);
                if (value4 != null) {
                    if (this.fontsAreRegistered) {
                        this.fontNameCache.put(string, value4);
                        return value4;
                    }
                    return value4;
                }
            }
            if (!this.loadedAllFonts) {
                if (FontUtilities.isLogging()) {
                    FontUtilities.getLogger().info("Load fonts looking for:" + str);
                }
                this.loadFonts();
                this.loadedAllFonts = true;
                return this.findFont2D(str, i, n);
            }
            else if (!this.loadedAllFontFiles) {
                if (FontUtilities.isLogging()) {
                    FontUtilities.getLogger().info("Load font files looking for:" + str);
                }
                this.loadFontFiles();
                this.loadedAllFontFiles = true;
                return this.findFont2D(str, i, n);
            }
            else {
                Font2D font2DAllLocales;
                if ((font2DAllLocales = this.findFont2DAllLocales(str, i)) != null) {
                    this.fontNameCache.put(string, font2DAllLocales);
                    return font2DAllLocales;
                }
                if (FontUtilities.isWindows) {
                    String fallbackFamilyName = this.getFontConfiguration().getFallbackFamilyName(str, null);
                    if (fallbackFamilyName != null) {
                        Font2D font2D5 = this.findFont2D(fallbackFamilyName, i, n);
                        this.fontNameCache.put(string, font2D5);
                        return font2D5;
                    }
                }
                else {
                    if (lowerCase.equals("timesroman")) {
                        Font2D font2D4 = this.findFont2D("serif", i, n);
                        this.fontNameCache.put(string, font2D4);
                        return font2D4;
                    }
                    if (lowerCase.equals("helvetica")) {
                        Font2D font2D3 = this.findFont2D("sansserif", i, n);
                        this.fontNameCache.put(string, font2D3);
                        return font2D3;
                    }
                    if (lowerCase.equals("courier")) {
                        Font2D font2D2 = this.findFont2D("monospaced", i, n);
                        this.fontNameCache.put(string, font2D2);
                        return font2D2;
                    }
                }
                if (FontUtilities.isLogging()) {
                    FontUtilities.getLogger().info("No font found for:" + str);
                }
                switch (n) {
                    case 1: {
                        return this.getDefaultPhysicalFont();
                    }
                    case 2: {
                        return this.getDefaultLogicalFont(i);
                    }
                    default: {
                        return null;
                    }
                }
            }
        }
    }
    
    public boolean usePlatformFontMetrics() {
        return this.usePlatformFontMetrics;
    }
    
    public int getNumFonts() {
        return this.physicalFonts.size() + this.maxCompFont;
    }
    
    private static boolean fontSupportsEncoding(Font font, String s) {
        return FontUtilities.getFont2D(font).supportsEncoding(s);
    }
    
    protected abstract String getFontPath(boolean p0);
    
    @Override
    public Font2D createFont2D(final File e, int n, boolean b, final CreatedFontTracker createdFontTracker) throws FontFormatException {
        String path = e.getPath();
        TrueTypeFont trueTypeFont;
        try {
            switch (n) {
                case 0: {
                    trueTypeFont = new TrueTypeFont(path, null, 0, true);
                    break;
                }
                case 1: {
                    trueTypeFont = new Type1Font(path, null, b);
                    break;
                }
                default: {
                    throw new FontFormatException("Unrecognised Font Format");
                }
            }
        }
        catch (FontFormatException ex) {
            if (b) {
                AccessController.doPrivileged(new PrivilegedAction() {
                    @Override
                    public Object run() {
                        if (createdFontTracker != null) {
                            createdFontTracker.subBytes((int)e.length());
                        }
                        e.delete();
                        return null;
                    }
                });
            }
            throw ex;
        }
        if (b) {
            trueTypeFont.setFileToRemove(e, createdFontTracker);
            synchronized (FontManager.class) {
                if (this.tmpFontFiles == null) {
                    this.tmpFontFiles = new Vector();
                }
                this.tmpFontFiles.add(e);
                if (this.fileCloser == null) {
                    AccessController.doPrivileged(()/*  */ -> {
                        SunFontManager.8 o = new Runnable() {
                            @Override
                            public void run() {
                                AccessController.doPrivileged(new PrivilegedAction() {
                                    @Override
                                    public Object run() {
                                        for (int i = 0; i < 20; ++i) {
                                            if (SunFontManager.this.fontFileCache[i] != null) {
                                                try {
                                                    SunFontManager.this.fontFileCache[i].close();
                                                }
                                                catch (Exception ex) {}
                                            }
                                        }
                                        if (SunFontManager.this.tmpFontFiles != null) {
                                            File[] array = (File[])SunFontManager.this.tmpFontFiles.toArray(new File[SunFontManager.this.tmpFontFiles.size()]);
                                            for (int j = 0; j < array.length; ++j) {
                                                try {
                                                    array[j].delete();
                                                }
                                                catch (Exception ex2) {}
                                            }
                                        }
                                        return null;
                                    }
                                });
                            }
                        };
                        (this.fileCloser = new Thread(ThreadGroupUtils.getRootThreadGroup(), task)).setContextClassLoader(null);
                        Runtime.getRuntime().addShutdownHook(this.fileCloser);
                        return null;
                    });
                }
            }
        }
        return trueTypeFont;
    }
    
    public synchronized String getFullNameByFileName(String anObject) {
        PhysicalFont[] physicalFonts = this.getPhysicalFonts();
        int i = 0;
        while (i < physicalFonts.length) {
            if (physicalFonts[i].platName.equals(anObject)) {
                return physicalFonts[i].getFontName(null);
            }
            ++i;
        }
        return null;
    }
    
    @Override
    public synchronized void deRegisterBadFont(Font2D obj) {
        if (!(obj instanceof PhysicalFont)) {
            return;
        }
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().severe("Deregister bad font: " + obj);
        }
        this.replaceFont((PhysicalFont)obj, this.getDefaultPhysicalFont());
        return;
    }
    
    public synchronized void replaceFont(PhysicalFont obj, PhysicalFont physicalFont) {
        if (obj.handle.font2D != obj) {
            return;
        }
        if (obj == physicalFont) {
            if (FontUtilities.isLogging()) {
                FontUtilities.getLogger().severe("Can't replace bad font with itself " + obj);
            }
            PhysicalFont[] physicalFonts = this.getPhysicalFonts();
            int i = 0;
            while (i < physicalFonts.length) {
                if (physicalFonts[i] != physicalFont) {
                    physicalFont = physicalFonts[i];
                    break;
                }
                ++i;
            }
            if (obj == physicalFont) {
                if (FontUtilities.isLogging()) {
                    FontUtilities.getLogger().severe("This is bad. No good physicalFonts found.");
                }
                return;
            }
        }
        obj.handle.font2D = physicalFont;
        this.physicalFonts.remove(obj.fullName);
        this.fullNameToFont.remove(obj.fullName.toLowerCase(Locale.ENGLISH));
        FontFamily.remove(obj);
        if (this.localeFullNamesToFont != null) {
            Map.Entry[] array = (Map.Entry[])(Map.Entry[])this.localeFullNamesToFont.entrySet().toArray(new Map.Entry[0]);
            for (int j = 0; j < array.length; ++j) {
                if (array[j].getValue() == obj) {
                    try {
                        array[j].setValue(physicalFont);
                    }
                    catch (Exception ex) {
                        this.localeFullNamesToFont.remove(array[j].getKey());
                    }
                }
            }
        }
        for (int k = 0; k < this.maxCompFont; ++k) {
            if (physicalFont.getRank() > 2) {
                this.compFonts[k].replaceComponentFont(obj, physicalFont);
            }
        }
        return;
    }
    
    private synchronized void loadLocaleNames() {
        if (this.localeFullNamesToFont != null) {
            return;
        }
        this.localeFullNamesToFont = new HashMap();
        Font2D[] registeredFonts = this.getRegisteredFonts();
        for (int i = 0; i < registeredFonts.length; ++i) {
            if ((registeredFonts[i] instanceof TrueTypeFont)) {
                TrueTypeFont value = (TrueTypeFont)registeredFonts[i];
                String[] allFullNames = value.getAllFullNames();
                for (int j = 0; j < allFullNames.length; ++j) {
                    this.localeFullNamesToFont.put(allFullNames[j], value);
                }
                FontFamily family = FontFamily.getFamily(value.familyName);
                if (family != null) {
                    FontFamily.addLocaleNames(family, value.getAllFamilyNames());
                }
            }
        }
        return;
    }
    
    private Font2D findFont2DAllLocales(String s, int n) {
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().info("Searching localised font names for:" + s);
        }
        if (this.localeFullNamesToFont == null) {
            this.loadLocaleNames();
        }
        String lowerCase = s.toLowerCase();
        Font2D font2D = null;
        FontFamily localeFamily = FontFamily.getLocaleFamily(lowerCase);
        if (localeFamily != null) {
            font2D = localeFamily.getFont(n);
            if (font2D == null) {
                font2D = localeFamily.getClosestStyle(n);
            }
            if (font2D != null) {
                return font2D;
            }
        }
        synchronized (this) {
            font2D = (Font2D)this.localeFullNamesToFont.get(s);
        }
        if (font2D != null) {
            if (font2D.style == n || n == 0) {
                return font2D;
            }
            FontFamily family = FontFamily.getFamily(font2D.getFamilyName(null));
            if (family != null) {
                Font2D font = family.getFont(n);
                if (font != null) {
                    return font;
                }
                Font2D closestStyle = family.getClosestStyle(n);
                if (closestStyle != null) {
                    if (!closestStyle.canDoStyle(n)) {
                        closestStyle = null;
                        return closestStyle;
                    }
                    return closestStyle;
                }
            }
        }
        return font2D;
    }
    
    public boolean maybeUsingAlternateCompositeFonts() {
        return this._usingAlternateComposites || this._usingPerAppContextComposites;
    }
    
    public boolean usingAlternateCompositeFonts() {
        return this._usingAlternateComposites || (this._usingPerAppContextComposites && AppContext.getAppContext().get(CompositeFont.class) != null);
    }
    
    private static boolean maybeMultiAppContext() {
        return (boolean)AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                return new Boolean(System.getSecurityManager() instanceof AppletSecurity);
            }
        });
    }
    
    @Override
    public synchronized void useAlternateFontforJALocales() {
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().info("Entered useAlternateFontforJALocales().");
        }
        if (!FontUtilities.isWindows) {
            return;
        }
        if (!maybeMultiAppContext()) {
            SunFontManager.gAltJAFont = true;
        }
        else {
            AppContext.getAppContext().put(SunFontManager.altJAFontKey, SunFontManager.altJAFontKey);
        }
        return;
    }
    
    public boolean usingAlternateFontforJALocales() {
        if (!maybeMultiAppContext()) {
            return SunFontManager.gAltJAFont;
        }
        return AppContext.getAppContext().get(SunFontManager.altJAFontKey) == SunFontManager.altJAFontKey;
    }
    
    @Override
    public synchronized void preferLocaleFonts() {
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().info("Entered preferLocaleFonts().");
        }
        if (!FontConfiguration.willReorderForStartupLocale()) {
            return;
        }
        if (!maybeMultiAppContext()) {
            if (this.gLocalePref) {
                return;
            }
            this.gLocalePref = true;
            this.createCompositeFonts(this.fontNameCache, this.gLocalePref, this.gPropPref);
            this._usingAlternateComposites = true;
        }
        else {
            AppContext appContext = AppContext.getAppContext();
            if (appContext.get(SunFontManager.localeFontKey) == SunFontManager.localeFontKey) {
                return;
            }
            appContext.put(SunFontManager.localeFontKey, SunFontManager.localeFontKey);
            int n = appContext.get(SunFontManager.proportionalFontKey) == SunFontManager.proportionalFontKey;
            ConcurrentHashMap<String, Font2D> value = new ConcurrentHashMap();
            appContext.put(CompositeFont.class, value);
            this.createCompositeFonts(value, this._usingPerAppContextComposites = true, (boolean)n);
        }
        return;
    }
    
    @Override
    public synchronized void preferProportionalFonts() {
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().info("Entered preferProportionalFonts().");
        }
        if (!FontConfiguration.hasMonoToPropMap()) {
            return;
        }
        if (!maybeMultiAppContext()) {
            if (this.gPropPref) {
                return;
            }
            this.gPropPref = true;
            this.createCompositeFonts(this.fontNameCache, this.gLocalePref, this.gPropPref);
            this._usingAlternateComposites = true;
        }
        else {
            AppContext appContext = AppContext.getAppContext();
            if (appContext.get(SunFontManager.proportionalFontKey) == SunFontManager.proportionalFontKey) {
                return;
            }
            appContext.put(SunFontManager.proportionalFontKey, SunFontManager.proportionalFontKey);
            int n = appContext.get(SunFontManager.localeFontKey) == SunFontManager.localeFontKey;
            ConcurrentHashMap<String, Font2D> value = new ConcurrentHashMap();
            appContext.put(CompositeFont.class, value);
            this.createCompositeFonts(value, (boolean)n, this._usingPerAppContextComposites = true);
        }
        return;
    }
    
    private static HashSet<String> getInstalledNames() {
        if (SunFontManager.installedNames == null) {
            Locale systemStartupLocale = getSystemStartupLocale();
            SunFontManager instance = getInstance();
            String[] installedFontFamilyNames = instance.getInstalledFontFamilyNames(systemStartupLocale);
            Font[] allInstalledFonts = instance.getAllInstalledFonts();
            HashSet<String> installedNames = new HashSet();
            for (int i = 0; i < installedFontFamilyNames.length; ++i) {
                installedNames.add(installedFontFamilyNames[i].toLowerCase(systemStartupLocale));
            }
            for (int j = 0; j < allInstalledFonts.length; ++j) {
                installedNames.add(allInstalledFonts[j].getFontName(systemStartupLocale).toLowerCase(systemStartupLocale));
            }
            SunFontManager.installedNames = installedNames;
        }
        return SunFontManager.installedNames;
    }
    
    @Override
    public boolean registerFont(Font font) {
        if (font == null) {
            return false;
        }
        synchronized (SunFontManager.regFamilyKey) {
            if (this.createdByFamilyName == null) {
                this.createdByFamilyName = new Hashtable();
                this.createdByFullName = new Hashtable();
            }
        }
        if (!FontAccess.getFontAccess().isCreatedFont(font)) {
            return false;
        }
        HashSet<String> installedNames = getInstalledNames();
        Locale systemStartupLocale = getSystemStartupLocale();
        String lowerCase = font.getFamily(systemStartupLocale).toLowerCase();
        String lowerCase2 = font.getFontName(systemStartupLocale).toLowerCase();
        if (installedNames.contains(lowerCase) || installedNames.contains(lowerCase2)) {
            return false;
        }
        Hashtable<String, FontFamily> createdByFamilyName;
        Hashtable<String, Font2D> createdByFullName;
        if (!maybeMultiAppContext()) {
            createdByFamilyName = this.createdByFamilyName;
            createdByFullName = this.createdByFullName;
            this.fontsAreRegistered = true;
        }
        else {
            AppContext appContext = AppContext.getAppContext();
            createdByFamilyName = (Hashtable)appContext.get(SunFontManager.regFamilyKey);
            createdByFullName = (Hashtable)appContext.get(SunFontManager.regFullNameKey);
            if (createdByFamilyName == null) {
                createdByFamilyName = new Hashtable();
                createdByFullName = new Hashtable();
                appContext.put(SunFontManager.regFamilyKey, createdByFamilyName);
                appContext.put(SunFontManager.regFullNameKey, createdByFullName);
            }
            this.fontsAreRegisteredPerAppContext = true;
        }
        Font2D font2D = FontUtilities.getFont2D(font);
        int style = font2D.getStyle();
        FontFamily value = (Object)createdByFamilyName.get(lowerCase);
        if (value == null) {
            value = new FontFamily(font.getFamily(systemStartupLocale));
            createdByFamilyName.put(lowerCase, value);
        }
        if (this.fontsAreRegistered) {
            this.removeFromCache(value.getFont(0));
            this.removeFromCache(value.getFont(1));
            this.removeFromCache(value.getFont(2));
            this.removeFromCache(value.getFont(3));
            this.removeFromCache((Font2D)createdByFullName.get(lowerCase2));
        }
        value.setFont(font2D, style);
        createdByFullName.put(lowerCase2, font2D);
        return true;
    }
    
    private void removeFromCache(Font2D font2D) {
        if (font2D == null) {
            return;
        }
        String[] array = (String[])(String[])this.fontNameCache.keySet().toArray(SunFontManager.STR_ARRAY);
        for (int i = 0; i < array.length; ++i) {
            if (this.fontNameCache.get(array[i]) == font2D) {
                this.fontNameCache.remove(array[i]);
            }
        }
        return;
    }
    
    @Override
    public TreeMap<String, String> getCreatedFontFamilyNames() {
        Hashtable<Object, Object> createdByFamilyName;
        if (this.fontsAreRegistered) {
            createdByFamilyName = this.createdByFamilyName;
        }
        else if (this.fontsAreRegisteredPerAppContext) {
            createdByFamilyName = (Hashtable)AppContext.getAppContext().get(SunFontManager.regFamilyKey);
        }
        else {
            return null;
        }
        Locale systemStartupLocale = getSystemStartupLocale();
        synchronized (createdByFamilyName) {
            TreeMap o = new TreeMap();
            for (FontFamily fontFamily : createdByFamilyName.values()) {
                Font2D font2D = fontFamily.getFont(0);
                if (font2D == null) {
                    font2D = fontFamily.getClosestStyle(0);
                }
                String familyName = font2D.getFamilyName(systemStartupLocale);
                o.put(familyName.toLowerCase(systemStartupLocale), familyName);
            }
            return o;
        }
    }
    
    @Override
    public Font[] getCreatedFonts() {
        Hashtable<Object, Object> createdByFullName;
        if (this.fontsAreRegistered) {
            createdByFullName = this.createdByFullName;
        }
        else if (this.fontsAreRegisteredPerAppContext) {
            createdByFullName = (Hashtable)AppContext.getAppContext().get(SunFontManager.regFullNameKey);
        }
        else {
            return null;
        }
        Locale systemStartupLocale = getSystemStartupLocale();
        synchronized (createdByFullName) {
            Font[] array = new Font[createdByFullName.size()];
            int n = 0;
            Iterator<Object> iterator = createdByFullName.values().iterator();
            while (iterator.hasNext()) {
                array[n++] = new Font(((Font2D)iterator.next()).getFontName(systemStartupLocale), 0, 1);
            }
            return array;
        }
    }
    
    protected String[] getPlatformFontDirs(boolean b) {
        if (this.pathDirs != null) {
            return this.pathDirs;
        }
        StringTokenizer stringTokenizer = new StringTokenizer(this.getPlatformFontPath(b), File.pathSeparator);
        ArrayList<String> list = new ArrayList();
        try {
            while (stringTokenizer.hasMoreTokens()) {
                list.add(stringTokenizer.nextToken());
            }
        }
        catch (NoSuchElementException ex) {}
        return this.pathDirs = (String[])list.toArray(new String[0]);
    }
    
    protected abstract String[] getDefaultPlatformFont();
    
    private void addDirFonts(String str, File parent, FilenameFilter filter, int n, boolean b, int n2, boolean b2, boolean b3) {
        String[] list = parent.list(filter);
        if (list == null || list.length == 0) {
            return;
        }
        String[] array = new String[list.length];
        String[][] array2 = new String[list.length][];
        int n3 = 0;
        for (int i = 0; i < list.length; ++i) {
            File o = new File(parent, list[i]);
            String str2 = null;
            if (b3) {
                try {
                    str2 = o.getCanonicalPath();
                }
                catch (IOException ex) {}
            }
            if (str2 == null) {
                str2 = str + File.separator + list[i];
            }
            if (!this.registeredFontFiles.contains(str2)) {
                if (this.badFonts != null && this.badFonts.contains(str2)) {
                    if (FontUtilities.debugFonts()) {
                        FontUtilities.getLogger().warning("skip bad font " + str2);
                    }
                }
                else {
                    this.registeredFontFiles.add(str2);
                    if (FontUtilities.debugFonts() && FontUtilities.getLogger().isLoggable(PlatformLogger.Level.INFO)) {
                        String string = "Registering font " + str2;
                        String[] nativeNames = this.getNativeNames(str2, null);
                        String s;
                        if (nativeNames == null) {
                            s = string + " with no native name";
                        }
                        else {
                            s = string + " with native name(s) " + nativeNames[0];
                            for (int j = 1; j < nativeNames.length; ++j) {
                                s = s + ", " + nativeNames[j];
                            }
                        }
                        FontUtilities.getLogger().info(s);
                    }
                    array[n3] = str2;
                    array2[n3++] = this.getNativeNames(str2, null);
                }
            }
        }
        this.registerFonts(array, array2, n3, n, b, n2, b2);
        return;
    }
    
    protected String[] getNativeNames(String s, String s2) {
        return null;
    }
    
    protected String getFileNameFromPlatformName(String platformName) {
        return this.fontConfig.getFileNameFromPlatformName(platformName);
    }
    
    @Override
    public FontConfiguration getFontConfiguration() {
        return this.fontConfig;
    }
    
    public String getPlatformFontPath(boolean b) {
        if (this.fontPath == null) {
            this.fontPath = this.getFontPath(b);
        }
        return this.fontPath;
    }
    
    public static boolean isOpenJDK() {
        return FontUtilities.isOpenJDK;
    }
    
    protected void loadFonts() {
        if (this.discoveredAllFonts) {
            return;
        }
        synchronized (this) {
            if (FontUtilities.debugFonts()) {
                Thread.dumpStack();
                FontUtilities.getLogger().info("SunGraphicsEnvironment.loadFonts() called");
            }
            this.initialiseDeferredFonts();
            AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    if (SunFontManager.this.fontPath == null) {
                        SunFontManager.this.fontPath = SunFontManager.this.getPlatformFontPath(SunFontManager.noType1Font);
                        SunFontManager.this.registerFontDirs(SunFontManager.this.fontPath);
                    }
                    if (SunFontManager.this.fontPath != null && !SunFontManager.this.gotFontsFromPlatform()) {
                        SunFontManager.access$1100(SunFontManager.this, SunFontManager.this.fontPath, false, 6, false, true);
                        SunFontManager.this.loadedAllFontFiles = true;
                    }
                    SunFontManager.this.registerOtherFontFiles(SunFontManager.this.registeredFontFiles);
                    SunFontManager.this.discoveredAllFonts = true;
                    return null;
                }
            });
        }
        return;
    }
    
    protected void registerFontDirs(String s) {
    }
    
    private void registerFontsOnPath(String str, boolean b, int n, boolean b2, boolean b3) {
        StringTokenizer stringTokenizer = new StringTokenizer(str, File.pathSeparator);
        try {
            while (stringTokenizer.hasMoreTokens()) {
                this.registerFontsInDir(stringTokenizer.nextToken(), b, n, b2, b3);
            }
        }
        catch (NoSuchElementException ex) {}
    }
    
    public void registerFontsInDir(String s) {
        this.registerFontsInDir(s, true, 2, true, false);
    }
    
    protected void registerFontsInDir(String pathname, boolean b, int n, boolean b2, boolean b3) {
        File file = new File(pathname);
        this.addDirFonts(pathname, file, SunFontManager.ttFilter, 0, b, (n == 6) ? 3 : n, b2, b3);
        this.addDirFonts(pathname, file, SunFontManager.t1Filter, 1, b, (n == 6) ? 4 : n, b2, b3);
    }
    
    protected void registerFontDir(String s) {
    }
    
    public synchronized String getDefaultFontFile() {
        if (this.defaultFontFileName == null) {
            this.initDefaultFonts();
        }
        return this.defaultFontFileName;
    }
    
    private void initDefaultFonts() {
        if (!isOpenJDK()) {
            this.defaultFontName = "Lucida Sans Regular";
            if (this.useAbsoluteFontFileNames()) {
                this.defaultFontFileName = SunFontManager.jreFontDirName + File.separator + "LucidaSansRegular.ttf";
            }
            else {
                this.defaultFontFileName = "LucidaSansRegular.ttf";
            }
        }
    }
    
    protected boolean useAbsoluteFontFileNames() {
        return true;
    }
    
    protected abstract FontConfiguration createFontConfiguration();
    
    public abstract FontConfiguration createFontConfiguration(boolean p0, boolean p1);
    
    public synchronized String getDefaultFontFaceName() {
        if (this.defaultFontName == null) {
            this.initDefaultFonts();
        }
        return this.defaultFontName;
    }
    
    public void loadFontFiles() {
        this.loadFonts();
        if (this.loadedAllFontFiles) {
            return;
        }
        synchronized (this) {
            if (FontUtilities.debugFonts()) {
                Thread.dumpStack();
                FontUtilities.getLogger().info("loadAllFontFiles() called");
            }
            AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    if (SunFontManager.this.fontPath == null) {
                        SunFontManager.this.fontPath = SunFontManager.this.getPlatformFontPath(SunFontManager.noType1Font);
                    }
                    if (SunFontManager.this.fontPath != null) {
                        SunFontManager.access$1100(SunFontManager.this, SunFontManager.this.fontPath, false, 6, false, true);
                    }
                    SunFontManager.this.loadedAllFontFiles = true;
                    return null;
                }
            });
        }
        return;
    }
    
    private void initCompositeFonts(FontConfiguration fontConfiguration, ConcurrentHashMap<String, Font2D> concurrentHashMap) {
        if (FontUtilities.isLogging()) {
            FontUtilities.getLogger().info("Initialising composite fonts");
        }
        int numberCoreFonts = fontConfiguration.getNumberCoreFonts();
        String[] platformFontNames = fontConfiguration.getPlatformFontNames();
        for (int i = 0; i < platformFontNames.length; ++i) {
            String anObject = platformFontNames[i];
            String fileNameFromPlatformName = this.getFileNameFromPlatformName(anObject);
            String[] nativeNames = null;
            if (fileNameFromPlatformName == null || fileNameFromPlatformName.equals(anObject)) {
                fileNameFromPlatformName = anObject;
            }
            else {
                if (i < numberCoreFonts) {
                    this.addFontToPlatformFontPath(anObject);
                }
                nativeNames = this.getNativeNames(fileNameFromPlatformName, anObject);
            }
            this.registerFontFile(fileNameFromPlatformName, nativeNames, 2, true);
        }
        this.registerPlatformFontsUsedByFontConfiguration();
        CompositeFontDescriptor[] get2DCompositeFontInfo = fontConfiguration.get2DCompositeFontInfo();
        for (int j = 0; j < get2DCompositeFontInfo.length; ++j) {
            CompositeFontDescriptor compositeFontDescriptor = get2DCompositeFontInfo[j];
            String[] componentFileNames = compositeFontDescriptor.getComponentFileNames();
            String[] componentFaceNames = compositeFontDescriptor.getComponentFaceNames();
            if (SunFontManager.missingFontFiles != null) {
                for (int k = 0; k < componentFileNames.length; ++k) {
                    if (SunFontManager.missingFontFiles.contains(componentFileNames[k])) {
                        componentFileNames[k] = this.getDefaultFontFile();
                        componentFaceNames[k] = this.getDefaultFontFaceName();
                    }
                }
            }
            if (concurrentHashMap != null) {
                registerCompositeFont(compositeFontDescriptor.getFaceName(), componentFileNames, componentFaceNames, compositeFontDescriptor.getCoreComponentCount(), compositeFontDescriptor.getExclusionRanges(), compositeFontDescriptor.getExclusionRangeLimits(), true, concurrentHashMap);
            }
            else {
                this.registerCompositeFont(compositeFontDescriptor.getFaceName(), componentFileNames, componentFaceNames, compositeFontDescriptor.getCoreComponentCount(), compositeFontDescriptor.getExclusionRanges(), compositeFontDescriptor.getExclusionRangeLimits(), true);
            }
            if (FontUtilities.debugFonts()) {
                FontUtilities.getLogger().info("registered " + compositeFontDescriptor.getFaceName());
            }
        }
    }
    
    protected void addFontToPlatformFontPath(String s) {
    }
    
    protected void registerFontFile(String s, String[] array, int n, boolean b) {
        if (this.registeredFontFiles.contains(s)) {
            return;
        }
        int n2;
        if (SunFontManager.ttFilter.accept(null, s)) {
            n2 = 0;
        }
        else if (SunFontManager.t1Filter.accept(null, s)) {
            n2 = 1;
        }
        else {
            n2 = 5;
        }
        this.registeredFontFiles.add(s);
        if (b) {
            this.registerDeferredFont(s, s, array, n2, false, n);
        }
        else {
            this.registerFontFile(s, array, n2, false, n);
        }
        return;
    }
    
    protected void registerPlatformFontsUsedByFontConfiguration() {
    }
    
    protected void addToMissingFontFileList(String e) {
        if (SunFontManager.missingFontFiles == null) {
            SunFontManager.missingFontFiles = new HashSet();
        }
        SunFontManager.missingFontFiles.add(e);
    }
    
    private boolean isNameForRegisteredFile(String s) {
        String fileNameForFontName = this.getFileNameForFontName(s);
        return fileNameForFontName != null && this.registeredFontFiles.contains(fileNameForFontName);
    }
    
    public void createCompositeFonts(ConcurrentHashMap<String, Font2D> concurrentHashMap, boolean b, boolean b2) {
        this.initCompositeFonts(this.createFontConfiguration(b, b2), concurrentHashMap);
    }
    
    @Override
    public Font[] getAllInstalledFonts() {
        if (this.allFonts == null) {
            this.loadFonts();
            TreeMap<String, Object> treeMap = new TreeMap();
            Font2D[] registeredFonts = this.getRegisteredFonts();
            for (int i = 0; i < registeredFonts.length; ++i) {
                if (!(registeredFonts[i] instanceof NativeFont)) {
                    treeMap.put(registeredFonts[i].getFontName(null), registeredFonts[i]);
                }
            }
            String[] fontNamesFromPlatform = this.getFontNamesFromPlatform();
            if (fontNamesFromPlatform != null) {
                for (int j = 0; j < fontNamesFromPlatform.length; ++j) {
                    if (!this.isNameForRegisteredFile(fontNamesFromPlatform[j])) {
                        treeMap.put(fontNamesFromPlatform[j], null);
                    }
                }
            }
            String[] array = null;
            if (treeMap.size() > 0) {
                array = new String[treeMap.size()];
                Object[] array2 = treeMap.keySet().toArray();
                for (int k = 0; k < array2.length; ++k) {
                    array[k] = (String)array2[k];
                }
            }
            Font[] allFonts = new Font[array.length];
            for (int l = 0; l < array.length; ++l) {
                allFonts[l] = new Font(array[l], 0, 1);
                Font2D font2D = (Font2D)treeMap.get(array[l]);
                if (font2D != null) {
                    FontAccess.getFontAccess().setFont2D(allFonts[l], font2D.handle);
                }
            }
            this.allFonts = allFonts;
        }
        Font[] array3 = new Font[this.allFonts.length];
        System.arraycopy(this.allFonts, 0, array3, 0, this.allFonts.length);
        return array3;
    }
    
    @Override
    public String[] getInstalledFontFamilyNames(Locale default1) {
        if (default1 == null) {
            default1 = Locale.getDefault();
        }
        if (this.allFamilies != null && this.lastDefaultLocale != null && default1.equals(this.lastDefaultLocale)) {
            String[] array3 = new String[this.allFamilies.length];
            System.arraycopy(this.allFamilies, 0, array3, 0, this.allFamilies.length);
            return array3;
        }
        TreeMap o = new TreeMap();
        String value = "Serif";
        o.put(value.toLowerCase(), value);
        String value2 = "SansSerif";
        o.put(value2.toLowerCase(), value2);
        String value3 = "Monospaced";
        o.put(value3.toLowerCase(), value3);
        String value4 = "Dialog";
        o.put(value4.toLowerCase(), value4);
        String value5 = "DialogInput";
        o.put(value5.toLowerCase(), value5);
        if (default1.equals(getSystemStartupLocale()) && this.getFamilyNamesFromPlatform(o, default1)) {
            this.getJREFontFamilyNames(o, default1);
        }
        else {
            this.loadFontFiles();
            PhysicalFont[] physicalFonts = this.getPhysicalFonts();
            for (int i = 0; i < physicalFonts.length; ++i) {
                if (!(physicalFonts[i] instanceof NativeFont)) {
                    String familyName = physicalFonts[i].getFamilyName(default1);
                    o.put(familyName.toLowerCase(default1), familyName);
                }
            }
        }
        this.addNativeFontFamilyNames(o, default1);
        String[] array = new String[o.size()];
        Object[] array2 = o.keySet().toArray();
        for (int j = 0; j < array2.length; ++j) {
            array[j] = (String)o.get(array2[j]);
        }
        if (default1.equals(Locale.getDefault())) {
            this.lastDefaultLocale = default1;
            System.arraycopy(array, 0, this.allFamilies = new String[array.length], 0, this.allFamilies.length);
            return array;
        }
        return array;
    }
    
    protected void addNativeFontFamilyNames(TreeMap<String, String> treeMap, Locale locale) {
    }
    
    public void register1dot0Fonts() {
        AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                SunFontManager.this.registerFontsInDir("/usr/openwin/lib/X11/fonts/Type1", true, 4, false, false);
                return null;
            }
        });
    }
    
    protected void getJREFontFamilyNames(TreeMap<String, String> treeMap, Locale locale) {
        this.registerDeferredJREFonts(SunFontManager.jreFontDirName);
        PhysicalFont[] physicalFonts = this.getPhysicalFonts();
        for (int i = 0; i < physicalFonts.length; ++i) {
            if (!(physicalFonts[i] instanceof NativeFont)) {
                String familyName = physicalFonts[i].getFamilyName(locale);
                treeMap.put(familyName.toLowerCase(locale), familyName);
            }
        }
    }
    
    private static Locale getSystemStartupLocale() {
        if (SunFontManager.systemLocale == null) {
            SunFontManager.systemLocale = (Locale)AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    String property = System.getProperty("file.encoding", "");
                    String property2 = System.getProperty("sun.jnu.encoding");
                    if (property2 != null && !property2.equals(property)) {
                        return Locale.ROOT;
                    }
                    return new Locale(System.getProperty("user.language", "en"), System.getProperty("user.country", ""), System.getProperty("user.variant", ""));
                }
            });
        }
        return SunFontManager.systemLocale;
    }
    
    void addToPool(FileFont fileFont) {
        FileFont fileFont2 = null;
        int n = -1;
        synchronized (this.fontFileCache) {
            int i = 0;
            while (i < 20) {
                if (this.fontFileCache[i] == fileFont) {
                    return;
                }
                if (this.fontFileCache[i] == null && n < 0) {
                    n = i;
                }
                ++i;
            }
            if (n >= 0) {
                this.fontFileCache[n] = fileFont;
                return;
            }
            fileFont2 = this.fontFileCache[this.lastPoolIndex];
            this.fontFileCache[this.lastPoolIndex] = fileFont;
            this.lastPoolIndex = (this.lastPoolIndex + 1) % 20;
        }
        if (fileFont2 != null) {
            fileFont2.close();
        }
    }
    
    protected FontUIResource getFontConfigFUIR(String name, int style, int size) {
        return new FontUIResource(name, style, size);
    }
    
    static void access$200() {
        initIDs();
    }
    
    static void access$700(SunFontManager sunFontManager, FontConfiguration fontConfiguration, ConcurrentHashMap concurrentHashMap) {
        sunFontManager.initCompositeFonts(fontConfiguration, concurrentHashMap);
    }
    
    static void access$1100(SunFontManager sunFontManager, String s, boolean b, int n, boolean b2, boolean b3) {
        sunFontManager.registerFontsOnPath(s, b, n, b2, b3);
    }
    
    static {
        SunFontManager.missingFontFiles = null;
        ttFilter = new SunFontManager.TTFilter();
        t1Filter = new SunFontManager.T1Filter();
        SunFontManager.STR_ARRAY = new String[0];
        AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                FontManagerNativeLibrary.load();
                SunFontManager.access$200();
                switch (StrikeCache.nativeAddressSize) {
                    case 8: {
                        SunFontManager.longAddresses = true;
                        break;
                    }
                    case 4: {
                        SunFontManager.longAddresses = false;
                        break;
                    }
                    default: {
                        throw new RuntimeException("Unexpected address size");
                    }
                }
                SunFontManager.noType1Font = "true".equals(System.getProperty("sun.java2d.noType1Font"));
                SunFontManager.jreLibDirName = System.getProperty("java.home", "") + File.separator + "lib";
                SunFontManager.jreFontDirName = SunFontManager.jreLibDirName + File.separator + "fonts";
                File file = new File(SunFontManager.jreFontDirName + File.separator + "LucidaSansRegular.ttf");
                return null;
            }
        });
        altJAFontKey = new Object();
        localeFontKey = new Object();
        proportionalFontKey = new Object();
        SunFontManager.gAltJAFont = false;
        SunFontManager.installedNames = null;
        regFamilyKey = new Object();
        regFullNameKey = new Object();
        SunFontManager.systemLocale = null;
    }
    
    private static class TTFilter implements FilenameFilter
    {
        @Override
        public boolean accept(File file, String s) {
            int n = s.length() - 4;
            return n > 0 && (s.startsWith(".ttf", n) || s.startsWith(".TTF", n) || s.startsWith(".ttc", n) || s.startsWith(".TTC", n) || s.startsWith(".otf", n) || s.startsWith(".OTF", n));
        }
    }
    
    private static class T1Filter implements FilenameFilter
    {
        @Override
        public boolean accept(File file, String s) {
            if (SunFontManager.noType1Font) {
                return false;
            }
            int n = s.length() - 4;
            return n > 0 && (s.startsWith(".pfa", n) || s.startsWith(".pfb", n) || s.startsWith(".PFA", n) || s.startsWith(".PFB", n));
        }
    }
    
    private static class TTorT1Filter implements FilenameFilter
    {
        @Override
        public boolean accept(File file, String s) {
            int n = s.length() - 4;
            return n > 0 && (s.startsWith(".ttf", n) || s.startsWith(".TTF", n) || s.startsWith(".ttc", n) || s.startsWith(".TTC", n) || s.startsWith(".otf", n) || s.startsWith(".OTF", n) || (!SunFontManager.noType1Font && (s.startsWith(".pfa", n) || s.startsWith(".pfb", n) || s.startsWith(".PFA", n) || s.startsWith(".PFB", n))));
        }
    }
    
    private static final class FontRegistrationInfo
    {
        String fontFilePath;
        String[] nativeNames;
        int fontFormat;
        boolean javaRasterizer;
        int fontRank;
        
        FontRegistrationInfo(String fontFilePath, String[] nativeNames, int fontFormat, boolean javaRasterizer, int fontRank) {
            this.fontFilePath = fontFilePath;
            this.nativeNames = nativeNames;
            this.fontFormat = fontFormat;
            this.javaRasterizer = javaRasterizer;
            this.fontRank = fontRank;
        }
    }
    
    public static class FamilyDescription
    {
        public String familyName;
        public String plainFullName;
        public String boldFullName;
        public String italicFullName;
        public String boldItalicFullName;
        public String plainFileName;
        public String boldFileName;
        public String italicFileName;
        public String boldItalicFileName;
    }
}
